package service

import (
	"context"

	"git.ego.cloudns.be/egommerce/api-entities/catalog/model"
	"git.ego.cloudns.be/egommerce/catalog-service/internal/event"

	"git.ego.cloudns.be/egommerce/go-api-pkg/fluentd"
	"git.ego.cloudns.be/egommerce/go-api-pkg/rabbitmq"

	"github.com/georgysavva/scany/v2/pgxscan"
	"github.com/jackc/pgx/v5/pgxpool"
	amqp "github.com/rabbitmq/amqp091-go"
)

const (
	BasketIdCookieName = "basket_id"
)

type CatalogService struct {
	dbConn *pgxpool.Pool
	ebCh   *amqp.Channel
	log    *fluentd.Logger
}

func NewCatalogService(dbConn *pgxpool.Pool, chn *amqp.Channel, log *fluentd.Logger) *CatalogService {
	return &CatalogService{dbConn, chn, log}
}

func (s *CatalogService) GetProduct(ctx context.Context, reqID string, productID int) (*model.ProductModel, error) {
	product := new(model.ProductModel)
	err := pgxscan.Get(ctx, s.dbConn, product, `SELECT id, pid, name, price FROM catalog.product WHERE id=$1`, productID)
	if err != nil {
		return nil, err
	}

	return product, nil
}

func (s *CatalogService) GetProductList(ctx context.Context, reqID string, categoryID int) ([]*model.ProductModel, error) {
	products := []*model.ProductModel{}

	sql := `SELECT id, pid, name, price FROM catalog.product`
	rows, err := s.dbConn.Query(ctx, sql) //, categoryID
	if err != nil {
		return nil, err
	}

	if err := pgxscan.ScanAll(&products, rows); err != nil {
		return nil, err
	}

	return products, nil // FIXME
}

func (s *CatalogService) AddProductToBasket(reqID, basketID string, productID, qty int) error {
	s.log.Log("Adding product #%d to the basket #%s", productID, basketID)

	msg := &event.ProductAddedToBasket{ // FIXME:move level up (ui)??
		Event:     event.NewEvent("AddProductToBasket", reqID),
		BasketID:  basketID,
		ProductID: productID,
		Quantity:  qty,
	}
	rabbitmq.Publish(s.ebCh, "api-events", "catalog.basket.productAddedToBasket", msg)
	rabbitmq.Publish(s.ebCh, "api-events", "catalog.warehouse.productAddedToBasket", msg)

	return nil
}

func (s *CatalogService) RemoveProductFromBasket(reqID, basketID string, productID, qty int) error {
	s.log.Log("Removed product#%d from basket#%s", productID, basketID)

	msg := &event.ProductRemovedFromBasket{
		Event:     event.NewEvent("RemoveProductFromBasket", reqID),
		BasketID:  basketID,
		ProductID: productID,
		Quantity:  qty,
	}
	rabbitmq.Publish(s.ebCh, "api-events", "catalog.basket.productRemovedFromBasket", msg)
	rabbitmq.Publish(s.ebCh, "api-events", "catalog.warehouse.productRemovedFromBasket", msg)

	return nil
}
